@@ -1,7 +1,7 @@
 /*
  * event.c	Server event handling
  *
- * Version:	$Id: 44089021565f5d5a8d0032d39d3958f50bbd20a8 $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: 44089021565f5d5a8d0032d39d3958f50bbd20a8 $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -508,7 +508,7 @@ static void wait_for_child_to_die(void *ctx)
 	 *	up) OR there's still a child thread handling it, THEN
 	 *	delay some more.
 	 */
-	if ((request->child_state = REQUEST_QUEUED) ||
+	if ((request->child_state == REQUEST_QUEUED) ||
 	    (request->thread_id != NO_CHILD_THREAD)) {
 		/*
 		 *	Cap delay at max_request_time
@@ -905,7 +905,7 @@ static void check_for_zombie_home_server(REQUEST *request)
 		return;
 	}
 
-	mark_home_server_dead(home, &request->when);
+	mark_home_server_dead(home, &when);
 }
 
 static int proxy_to_virtual_server(REQUEST *request);
@@ -1103,8 +1103,22 @@ static void no_response_to_proxied_request(void *ctx)
 
 		post_proxy_fail_handler(request);
 	} else {
+		struct timeval when;
+
+		/*
+		 *	Do nothing, and let the request time out.
+		 */
 		rad_assert(request->ev == NULL);
-		request->child_state = REQUEST_RUNNING;
+
+		when = request->received;
+		when.tv_sec += request->root->max_request_time;
+
+		request->child_state = REQUEST_DONE;
+		request->next_when = when;
+		request->next_callback = cleanup_delay;
+
+		if (request->in_proxy_hash) remove_from_proxy_hash(request);
+
 		wait_a_bit(request);
 	}
 
@@ -2461,6 +2475,16 @@ static void request_post_handler(REQUEST *request)
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 		}
 
+		/*
+		 *	Do post-auth.  If it returns reject, then
+		 *	run Post-Auth-Type Reject.
+		 */
+		if (request->reply->code == PW_AUTHENTICATION_ACK) {
+			if (rad_postauth(request) == RLM_MODULE_REJECT) {
+				request->reply->code = PW_AUTHENTICATION_REJECT;
+			}
+		}
+		
 		/*
 		 *	Run rejected packets through
 		 *
@@ -2496,10 +2520,6 @@ static void request_post_handler(REQUEST *request)
 			}
 		}
 		
-		if (request->reply->code == PW_AUTHENTICATION_ACK) {
-			rad_postauth(request);
-		}
-		
 		/* FALL-THROUGH */
 
 #ifdef WITH_COA
@@ -2631,14 +2651,11 @@ static void received_retransmit(REQUEST *request, const RADCLIENT *client)
 	switch (request->child_state) {
 	case REQUEST_QUEUED:
 	case REQUEST_RUNNING:
-#ifdef WITH_PROXY
-	discard:
-#endif
 		radlog(L_ERR, "Discarding duplicate request from "
-		       "client %s port %d - ID: %u due to unfinished request %u",
+		       "client %s port %d - ID: %u due to unfinished request %u in component %s module %s.",
 		       client->shortname,
 		       request->packet->src_port,request->packet->id,
-		       request->number);
+		       request->number, request->component, request->module);
 		break;
 
 #ifdef WITH_PROXY
@@ -2663,7 +2680,12 @@ static void received_retransmit(REQUEST *request, const RADCLIENT *client)
 		 *	packets, this logic has to be fixed.
 		 */
 		if (request->packet->code != PW_AUTHENTICATION_REQUEST) {
-			goto discard;
+			radlog(L_ERR, "Discarding duplicate request from "
+			       "client %s port %d - ID: %u due to unfinished proxied request %u",
+			       client->shortname,
+			       request->packet->src_port,request->packet->id,
+			       request->number);
+			break;
 		}
 
 		check_for_zombie_home_server(request);
@@ -3228,9 +3250,6 @@ REQUEST *received_proxy_response(RADIUS_PACKET *packet)
 	switch (request->child_state) {
 	case REQUEST_QUEUED:
 	case REQUEST_RUNNING:
-		radlog(L_ERR, "Internal sanity check failed for child state");
-		/* FALL-THROUGH */
-
 	case REQUEST_REJECT_DELAY:
 	case REQUEST_CLEANUP_DELAY:
 	case REQUEST_DONE:
