@@ -1,7 +1,7 @@
 /*
  * rlm_eap.c  contains handles that are called from modules.
  *
- * Version:     $Id: e165b73752ebe6bdfe0dd7c472bf723b605e9fbf $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: e165b73752ebe6bdfe0dd7c472bf723b605e9fbf $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -732,6 +732,62 @@ static int eap_post_proxy(void *inst, REQUEST *request)
 }
 #endif
 
+static int eap_post_auth(void *instance, REQUEST *request)
+{
+	rlm_eap_t	*inst = instance;
+	VALUE_PAIR	*vp;
+	EAP_HANDLER	*handler;
+	eap_packet_t	*eap_packet;
+
+	/*
+	 *	Only build a failure message if something previously rejected the request
+	 */
+	vp = pairfind(request->config_items, PW_POSTAUTHTYPE);
+
+	/*
+	 *	Post-Auth-Type REJECT in dictionary.freeradius.internal
+	 */
+	if (!vp || (vp->vp_integer != 2)) return RLM_MODULE_NOOP;
+
+	if (!pairfind(request->packet->vps, PW_EAP_MESSAGE)) {
+		RDEBUG2("Request didn't contain an EAP-Message, not inserting EAP-Failure");
+		return RLM_MODULE_NOOP;
+	}
+
+	if (pairfind(request->reply->vps, PW_EAP_MESSAGE)) {
+		RDEBUG2("Reply already contained an EAP-Message, not inserting EAP-Failure");
+		return RLM_MODULE_NOOP;
+	}
+
+	eap_packet = eap_vp2packet(request->packet->vps);
+	if (!eap_packet) {
+		RDEBUG("Malformed EAP Message");
+		return RLM_MODULE_FAIL;
+	}
+
+	handler = eap_handler(inst, &eap_packet, request);
+	if (!handler) {
+		RDEBUG2("Failed to get handler, probably already removed, not inserting EAP-Failure");
+		return RLM_MODULE_NOOP;
+	}
+
+	RDEBUG2("Request was previously rejected, inserting EAP-Failure");
+	eap_fail(handler);
+	eap_handler_free(inst, handler);
+
+	/*
+	 * Make sure there's a message authenticator attribute in the response
+	 * RADIUS protocol code will calculate the correct value later...
+	 */
+	vp = pairfind(request->reply->vps, PW_MESSAGE_AUTHENTICATOR);
+	if (!vp) {
+		vp = pairmake("Message-Authenticator", "0x00", T_OP_EQ);
+		pairadd(&request->reply->vps, vp);
+	}
+
+	return RLM_MODULE_UPDATED;
+}
+
 /*
  *	The module name should be the only globally exported symbol.
  *	That is, everything else should be 'static'.
@@ -754,6 +810,6 @@ module_t rlm_eap = {
 #else
 		NULL,
 #endif
-		NULL			/* post-auth */
+		eap_post_auth  		/* post-auth */
 	},
 };
